// RunSimOpsWithJava.java - v1.3 - Flavio Cimolin, 05/06/2023
// ---------------------------------------------------
// Macro to extend the functionality of the Simulation Operations by inserting
// java macros inside the pipeline.
//
// To prescribe a Java macro execution, just insert a dummy Simulation Operation
// of type "Set Parameter", with the name:
//      Run Macro MyMacro.java
// (of course substituting MyMacro.java with the desired macro).
//
// To launch the Simulation Operation including the prescribed Java macros,
// just set it as active and play this macro instead of the standard toolbar
// button to launch the Simulation Operations.
//
// The "Play Through Here" functionality (useful for debugging purpose) can be
// achieved by creating a Tag called "STOP" and assigning it to the last
// Simulation Operation Block to be executed.
// 
// The Macro execution cannot be the first Simulation Operation. In case you need 
// to play a macro as first thing, insert another dummy Operation before.
// 
// Note: in case you are willing to stop the execution, you will need to stop
// the macro, not the solver.
//
package macro;

import java.io.*;
import star.automation.*;
import star.common.*;

public class RunSimOpsWithJava extends StarMacro {

    // Internal Variables
    Simulation sim;

    @Override
    public void execute() {

        sim = getActiveSimulation();

        // Get the active Simulation Operation
        SimDriverWorkflow simDriverWorkflow = (SimDriverWorkflow) sim.get(SimDriverWorkflowManager.class).getSelectedWorkflow();

        if (simDriverWorkflow == null) {
            writeOutput("No active Simulation Operation found. Please set an active Simulation Operation first.");
            return;
        }

        // Check presence of STOP Tag
        Tag stopTag = sim.get(TagManager.class).hasObject("STOP");
        if (stopTag != null) {
            writeOutput("STOP Tag Identified");
        }

        // Get the active block
        AutomationBlock activeBlock = simDriverWorkflow.getActiveBlock();

        /* When the Simulation Operation is reset there is no active block,
        hence the simDriverWorkflow object returns null. The same occurs at the
        end of the Simulation Operation pipeline.
        
        This means that the very first block cannot be identified by this Java
        macro: this is the reason why it is treated in a different way from
        all the other ones, and why it can't handle a macro execution.
         */
        if (activeBlock == null) {

            writeOutput("Executing first Simulation Operation (this can't be a macro).\n");
            simDriverWorkflow.executeOneBlock();
            writeOutput("Simulation Operation executed.");

            activeBlock = simDriverWorkflow.getActiveBlock();

        }

        // Loop over Simulation Operation blocks
        boolean stopFlag = false;
        while (activeBlock != null) {

            String blockName = activeBlock.getPresentationName();

            // Check if current Simulation operation has a STOP tag
            if (stopTag != null) {
                if (activeBlock.getTagGroup().has(stopTag)) {

                    writeOutput("STOP Tag found at Simulation Operation: " + activeBlock + "\n");
                    stopFlag = true;

                }
            }

            // Identify keyword for macro execution
            if (blockName.startsWith("Run Macro ")) {

                String macroName = blockName.substring(10).split(" ")[0];

                writeOutput("Macro Operation found: " + blockName);

                // Check macro file existence (it is checked first in the same folder of this macro,
                // then in the same folder of the Simulation)
                File file = new File(resolvePath(macroName));

                if (!file.exists()) {
                    file = new File(sim.getSessionDir() + File.separator + macroName);
                }

                if (file.exists()) {

                    writeOutput("Executing macro: " + macroName + "\n");

                    // Execute macro
                    StarScript starScript = new StarScript(getActiveRootObject(), file);
                    starScript.play();

                } else {

                    writeOutput("Macro file " + macroName + " not found. Continuing.");

                }

            }

            writeOutput("Executing Simulation Operation: " + activeBlock + "\n");
            simDriverWorkflow.executeOneBlock();
            writeOutput("Simulation Operation executed.");

            activeBlock = simDriverWorkflow.getActiveBlock();

            if (stopFlag) {

                writeOutput("Stopping Simulation Operation Pipeline.");
                break;

            }

        }
    }

    private void writeOutput(String text) {
        sim.println("\n*** SimOpsWithJava ***  " + text);
    }

}
